"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports.resetModels = exports.registerModel = exports.pushModel = exports.getModel = exports.getHandlerNames = exports.getHandler = exports.addHandler = void 0;

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

const handlers = {};

const addHandler = (name, data) => {
  if (handlers[name] !== undefined) {
    throw new Error(`[postgres] handler "${name}" already defined`);
  }

  handlers[name] = Object.assign({}, data, {
    models: {}
  });
};

exports.addHandler = addHandler;

const getHandler = name => {
  if (handlers[name] === undefined) {
    throw new Error(`[postgres] handler "${name}" does not exists`);
  }

  return handlers[name];
};

exports.getHandler = getHandler;

const pushModel = (conn, model) => {
  if (conn.models[model.name] !== undefined) {
    throw new Error(`[postgres] model "${model.name}" already defined in "${conn.name}"`);
  }

  conn.models[model.name] = model;
};

exports.pushModel = pushModel;

const getModel = (modelName, connectionName = null) => {
  const model = Object.keys(handlers).filter(item => connectionName ? item === connectionName : true).map(name => Object.values(handlers[name].models)).reduce((acc, models) => [...acc, ...models], []).find(item => item.name === modelName);

  if (!model) {
    const connStr = connectionName ? `${connectionName}` : 'ALL CONNECTIONS';
    throw new Error(`[postgres] model not found "${modelName}" in "${connStr}"`);
  }

  return model;
};

exports.getModel = getModel;

const registerModel = /*#__PURE__*/function () {
  var _ref = (0, _asyncToGenerator2.default)(function* (model, connectionName = 'default') {
    try {
      const conn = handlers[connectionName];
      const instance = yield model.init(conn.handler);
      pushModel(conn, instance);
      yield model.start(conn, getModel(model.name, conn.name));
    } catch (err) {
      throw new Error(`[postgres] register "${model.name}" model failed in "${connectionName}" - ${err.message}`);
    }
  });

  return function registerModel(_x) {
    return _ref.apply(this, arguments);
  };
}();

exports.registerModel = registerModel;

const getHandlerNames = () => Object.keys(handlers);

exports.getHandlerNames = getHandlerNames;

const resetModels = /*#__PURE__*/function () {
  var _ref2 = (0, _asyncToGenerator2.default)(function* (connectionName = 'default') {
    try {
      const conn = handlers[connectionName];
      const models = Object.keys(conn.models).map(name => conn.models[name]);
      const modelsToPopulate = models.filter(model => model.populate); // sync and populate the models

      yield Promise.all(models.map(model => conn.handler.query(`TRUNCATE ${model.tableName} RESTART IDENTITY CASCADE;`)));
      yield Promise.all(models.map(model => model.sync()));
      yield Promise.all(modelsToPopulate.map(model => model.populate()));
    } catch (err) {
      throw new Error(`[postgres] resetModels "${connectionName}" failed - ${err.message}`);
    }
  });

  return function resetModels() {
    return _ref2.apply(this, arguments);
  };
}();

exports.resetModels = resetModels;