"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports.default = void 0;

var _objectWithoutPropertiesLoose2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutPropertiesLoose"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _pause = _interopRequireDefault(require("@marcopeg/utils/lib/pause"));

var _logger = require("./logger");

var _conn = require("./conn");

const _excluded = ["schemas", "models", "maxAttempts", "attemptDelay"];

const establishConnection = /*#__PURE__*/function () {
  var _ref = (0, _asyncToGenerator2.default)(function* (conn, maxAttempts = 10, attemptDelay = 1000) {
    let attempts = 0;
    let lastErrorMSG = '';

    do {
      try {
        (0, _logger.logVerbose)(`[postgres]  Connection attempt ${attempts + 1}/${maxAttempts} to "${conn.name}"`);
        yield conn.handler.authenticate();
        return true;
      } catch (err) {
        attempts += 1;
        lastErrorMSG = err.message;
        (0, _logger.logError)(`[postgres] failed connection "${conn.name}" - ${err.message}`);
        (0, _logger.logDebug)(err);
        yield (0, _pause.default)(attemptDelay);
      }
    } while (attempts < maxAttempts);

    throw new Error(lastErrorMSG);
  });

  return function establishConnection(_x) {
    return _ref.apply(this, arguments);
  };
}();

const initModels = /*#__PURE__*/function () {
  var _ref2 = (0, _asyncToGenerator2.default)(function* (conn, models) {
    const promises = models.map(model => new Promise( /*#__PURE__*/function () {
      var _ref3 = (0, _asyncToGenerator2.default)(function* (resolve, reject) {
        try {
          (0, _logger.logVerbose)(`[posgres] init model "${model.name}" in "${conn.name}"`);
          const instance = yield model.init(conn.handler, conn.hooksContext);
          (0, _conn.pushModel)(conn, instance);
          resolve();
        } catch (err) {
          (0, _logger.logError)(`[postgres] failed to init model "${model.name}" in "${conn.name}" - ${err.message}`);
          (0, _logger.logDebug)(err);
          reject({
            model,
            err
          });
        }
      });

      return function (_x4, _x5) {
        return _ref3.apply(this, arguments);
      };
    }()));
    return Promise.all(promises);
  });

  return function initModels(_x2, _x3) {
    return _ref2.apply(this, arguments);
  };
}();

const startModels = /*#__PURE__*/function () {
  var _ref4 = (0, _asyncToGenerator2.default)(function* (conn, models) {
    const promises = models.map(model => new Promise( /*#__PURE__*/function () {
      var _ref5 = (0, _asyncToGenerator2.default)(function* (resolve, reject) {
        if (!model.start) {
          return resolve();
        }

        try {
          (0, _logger.logVerbose)(`[postgres] start model "${model.name}" in "${conn.name}"`);
          yield model.start(conn.handler, (0, _conn.getModel)(model.name, conn.name), conn.hooksContext);
          resolve();
        } catch (err) {
          (0, _logger.logError)(`[postgres] failed to start model "${model.name}" in "${conn.name}" - ${err.message}`);
          (0, _logger.logDebug)(err);
          reject({
            model,
            err
          });
        }
      });

      return function (_x8, _x9) {
        return _ref5.apply(this, arguments);
      };
    }()));
    return Promise.all(promises);
  });

  return function startModels(_x6, _x7) {
    return _ref4.apply(this, arguments);
  };
}();

var _default = /*#__PURE__*/function () {
  var _ref7 = (0, _asyncToGenerator2.default)(function* (_ref6) {
    let schemas = _ref6.schemas,
        models = _ref6.models,
        maxAttempts = _ref6.maxAttempts,
        attemptDelay = _ref6.attemptDelay,
        config = (0, _objectWithoutPropertiesLoose2.default)(_ref6, _excluded);
    (0, _logger.logInfo)('[postgres] start');
    const name = config.connectionName || 'default';
    const conn = (0, _conn.getHandler)(name);

    try {
      yield establishConnection(conn, maxAttempts, attemptDelay);
    } catch (err) {
      throw new Error(`[postgres] connection failed "${name}" - ${err.message}`);
    } // Allow to run free queries right after connection was established


    try {
      if (config.onConnection) {
        yield config.onConnection(conn);
      }
    } catch (err) {
      throw new Error(`[postgres] onConnection hook failed for "${name}" - ${err.message}`);
    } // Create custom schemas


    if (schemas) {
      const schemasP = schemas.map(schema => conn.handler.query(`CREATE SCHEMA IF NOT EXISTS ${schema};`));
      yield Promise.all(schemasP);
    } // Initialize models


    try {
      yield initModels(conn, models);
    } catch (err) {
      throw new Error(`[postgres] init models failed for "${name}": ${err.model.name}, ${err.err.message}`);
    }

    try {
      if (config.afterInitModels) {
        yield config.afterInitModels(conn);
      }
    } catch (err) {
      throw new Error(`[postgres] afterInitModels hook failed for "${name}" - ${err.message}`);
    }

    try {
      yield startModels(conn, models);
    } catch (err) {
      throw new Error(`[postgres] start models failed for "${name}": ${err.model.name}, ${err.err.message}`);
    }

    try {
      if (config.afterStartModels) {
        yield config.afterStartModels(conn);
      }
    } catch (err) {
      throw new Error(`[postgres] afterStartModels hook failed for "${name}" - ${err.message}`);
    }
  });

  return function (_x10) {
    return _ref7.apply(this, arguments);
  };
}();

exports.default = _default;